//
//  YPCrawlController.m
//  YPCrawler
//
//  Created by Yasir  on 15/01/13.
//  Copyright (c) 2013 YP. All rights reserved.
//

#import "YPCrawlController.h"
#import "YPCrawler.h"

@implementation YPCrawlController
@synthesize delegate;

-(id) initWithBaseURL:(NSString*) baseUrl {
	self = [super init];
	
	if(self) {
		_waitingURLsArray = [[NSMutableArray alloc] init];
		_baseURL = [baseUrl copy];
		[_waitingURLsArray addObject:_baseURL];
		
		_crawledURLsSet = [[NSMutableSet alloc] init];
		[_crawledURLsSet addObject:[_waitingURLsArray objectAtIndex:0]];
		
		_crawlerArray = [[NSMutableArray alloc] init];
		
		crawledURLCount = 0;
	}
		
	return self;
}

-(void) dealloc {
	
	if(_waitingURLsArray){
		[_waitingURLsArray release];
		_waitingURLsArray = nil;
	}
	
	if(_crawledURLsSet){
		[_crawledURLsSet release];
		_crawledURLsSet = nil;
	}
	
	if(_crawlerArray) {
		[_crawlerArray release];
		_crawlerArray = nil;
	}
	if(_baseURL) {
		[_baseURL release];
		_baseURL = nil;
	}
	[super dealloc];
}
-(void) schedule {
	
	while ([_waitingURLsArray count] > 0) {
		if (crawledURLCount > 100) {
			break;
		}
		NSString* topUrl = [[_waitingURLsArray objectAtIndex:0] retain];
		[_waitingURLsArray removeObjectAtIndex:0];
		
		YPCrawler *crawler = [[[YPCrawler alloc] initWithDelegate:self] autorelease];
		[_crawlerArray addObject:crawler];
		[crawler crawlURL:topUrl];
		
		[topUrl release];
	}
}

-(void) stop {
	for (YPCrawler* crawler in _crawlerArray) {
		[crawler cancelCrawl];
	}
	[_crawlerArray removeAllObjects];
	[_crawledURLsSet removeAllObjects];
	[_waitingURLsArray removeAllObjects];
}

#pragma mark -
#pragma YPCrawlerDelegate
-(void) URLCrawlled:(NSString*) url contentSize:(NSInteger) contentSize linksFound:(NSArray*) linksFound sender:(id) sender{
	
	[_crawlerArray removeObject:sender];
//	NSLog(@"======================================================================");
//	NSLog(@"count %d url %@, content size %d linkes %@",crawledURLCount,url, contentSize, linksFound);
	crawledURLCount++;
	
	if(crawledURLCount >= 100) {
			//cancel all crawling
		for (YPCrawler* crawler in _crawlerArray) {
			[crawler cancelCrawl];
		}
		[_crawlerArray removeAllObjects];
		[_crawledURLsSet removeAllObjects];
		[_waitingURLsArray removeAllObjects];
	}else {
		for (NSString *link  in linksFound) {
			if(![_crawledURLsSet containsObject:link]) {
				[_crawledURLsSet addObject:link];
				[_waitingURLsArray addObject:link];

			}
		}
		
		if(self.delegate && [self.delegate respondsToSelector:@selector(URLCrawlled:contentSize:)]) {
			[self.delegate URLCrawlled:url contentSize:contentSize];
		}
		[self schedule];
	}
	
}
-(void) crawlError:(NSString*) url sender:(id) sender{
	[_crawlerArray removeObject:sender];
	if(crawledURLCount < 1) {
		if(self.delegate && [self.delegate respondsToSelector:@selector(crawlError)]) {
			[self.delegate crawlError];
		}
	}
	
}
@end
