//
//  YPCrawler.m
//  YPCrawler
//
//  Created by Yasir  on 15/01/13.
//  Copyright (c) 2013 YP. All rights reserved.
//

#import "YPCrawler.h"

const NSString* kRegExForLinks = @"href=\"https?[\'\"]?([^\'\" >]+)";

@interface YPCrawler ()
-(NSArray*) processResponse:(id) sender;
@end

@implementation YPCrawler


-(id) initWithDelegate:(id) delegate{
	
	self = [super init];
	
	if(self) {
		_responseDate = nil;
		_delegate = delegate;
	}
	
	return self;
}

-(void) dealloc {
	if(_responseDate){
		[_responseDate release];
		_responseDate = nil;
	}
	
	if(_url){
		[_url release];
		_url = nil;
	}
	if(_urlConnection){
		[_urlConnection release];
		_urlConnection = nil
		;
	}
	[super dealloc];
}


-(void) crawlURL:(NSString*) url {
	
	_url = [url copy];
	_responseDate = nil;
	NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:_url]];
	_urlConnection = [[NSURLConnection connectionWithRequest:request delegate:self] retain];
	if(_urlConnection) {
		if(_responseDate){
			[_responseDate release];
			_responseDate = nil;
		}
		_responseDate = [[NSMutableData data] retain];
	}
}

-(void) cancelCrawl {
	_delegate = nil;
	[_urlConnection cancel];
}
/*
 * this function take the data input and extract all link from the content.
 */

-(NSArray*) processResponse:(NSString*) responseData {
	NSError *error = nil;
	NSMutableArray *linkArray = nil;
	NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:kRegExForLinks options:0 error:&error];
	
	if (!error) {
		NSArray *results = [regex matchesInString:responseData options:0 range:NSMakeRange(0, responseData.length)];
		
		linkArray = [[[NSMutableArray alloc] init] autorelease];
		for (NSTextCheckingResult *entry in results)
		{
			NSString *text = [responseData substringWithRange:entry.range];
			[linkArray addObject:[text substringFromIndex:6]];
		}
	}
	
	return linkArray;
}

#pragma mark -
#pragma mark NSURLConnectionDelegate

-(void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	if(_delegate && [_delegate respondsToSelector:@selector(crawlError:sender:)]) {
		[_delegate crawlError:_url sender:self];
	}
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
	NSString* resStr = [[[NSString alloc] initWithData:_responseDate encoding:NSUTF8StringEncoding] autorelease];
		// this need to done in GCD
	if(resStr.length > 0) {
		
		dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0ul);
		
			// parse and extract all links in the async
		dispatch_async(queue, ^{
			NSArray *links = [self processResponse:resStr];
			
				// get back to UI thead
			dispatch_sync(dispatch_get_main_queue(), ^{
				
				if(_delegate && [_delegate respondsToSelector:@selector(URLCrawlled:contentSize:linksFound:sender:)]) {
					[_delegate URLCrawlled:_url contentSize:resStr.length linksFound:links sender:self];
				}
			});
		});
		
	}else if(_delegate && [_delegate respondsToSelector:@selector(crawlError:sender:)]) {
		[_delegate crawlError:_url sender:self];
	}
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response{
	[_responseDate setLength:0];
}
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[_responseDate appendData:data];
}
@end
